# frozen_string_literal: true

module Gdas
  module Clients
    class RefreshCredentialsProvider
      include ::Aws::RefreshingCredentials

      def initialize(customer_id, account_id, context: {})
        @customer_id = customer_id
        @aws_account_id = account_id
        @context = context

        @credentials_service = ImportRegistry.resolve('credentials')
        @logger = ImportRegistry.resolve('logger')
        @runtime = ImportRegistry.resolve('runtime')

        begin
          # ::Aws::RefreshingCredentials makes the call to refresh method below - catch and record exceptions here
          super() # Explicitly send no arguments to match.
        rescue => ex
          tags = %W(error:#{ex.class.name.gsub('::', '.')})
          ::ImportRegistry.resolve('statsd').increment 'credentials.initialization_failed', tags: tags
          raise
        end
      end

      # Called to by AWS SDK clients when near expiration
      def refresh
        @logger.debug "[Credentials Provider] Refreshing AWS Credentials"

        creds = @credentials_service.aws(@customer_id, @aws_account_id, context: @context)

        # #######################################################################
        # If this really is a bad account, then our account watcher will remove its schedule shortly

        # If we get nothing back, for whatever reason, then fail.
        raise ::Platform::NonRetryableError unless creds

        # It switched from accepted to something else in the past 15min since last account watcher refresh
        if creds.credentials_status != 'accepted'
          @logger.warn "AWS Account credentials are not accepted: #{@customer_id}:#{@aws_account_id}"
          raise ::Platform::NonRetryableError
        end

        # A little extra precaution, because blank values result in default behaviour of aws-sdk gem
        if creds.access_key.blank? || creds.secret_key.blank?
          # Due to bug: https://cloudhealthtech.atlassian.net/browse/ENG-7946
          # Even when that bug is fixed, this code *MUST* remain
          @logger.error "AWS Account credentials have blank values: #{@customer_id}:#{@aws_account_id}, creds response: #{creds.to_json}"
          ::ImportRegistry.resolve('statsd').increment 'credentials.credentials_blank_values'
          # This no longer disables collection
          raise ::Platform::CredentialsNotFoundError
        end

        @access_key_id = creds.access_key
        @secret_access_key = creds.secret_key
        @expiration = if creds.expiration.present?
                        Time.parse(creds.expiration)
                      else
                        nil
                      end

        if creds.auth_type == ::AwsAccount::AUTH_BY_ASSUMING_ROLE
          @session_token = creds.session_token
        end

        @credentials = ::Aws::Credentials.new(@access_key_id, @secret_access_key, @session_token)
      rescue ::Errno::ECONNREFUSED
        raise unless @runtime.sandbox?
        @logger.error 'Unable to connect to credentials service.'
        raise ::Platform::NonRetryableError
      end
    end
  end
end
