require 'memoist'

module Gdas
  # Cache of common assets
  # You'll see a pattern used throughout to call the memoized function again with `true`
  # This forces a refresh of the cache if that asset is not found. This is important because
  # when we add new regions or az's, they won't be in the existing cache, but we need them to work.
  class CommonAssets
    extend ::Memoist

    def initialize
      @ttls = {}
    end

    def flush
      @ttls = {}
      flush_cache # clear memoize
    end

    def data_store(id_or_name)
      if id_or_name.is_a?(Numeric) || id_or_name.to_i.positive?
        data_stores_by_id[id_or_name]
      else
        data_stores_by_name[id_or_name]
      end
    end

    def data_stores_by_name
      ::Multidb.use(:master) { ::DataStore.all.each_with_object({}) { |r, h| h[r.name] = r } }
    end

    memoize :data_stores_by_name

    def data_stores_by_id
      ::Multidb.use(:master) { ::DataStore.all.each_with_object({}) { |r, h| h[r.id] = r } }
    end

    memoize :data_stores_by_id

    def regions_by_name
      ::Multidb.use(:master) { ::AwsRegion.includes(:aws_availability_zones).each_with_object({}) { |r, h| h[r.name] = r } }
    end

    memoize :regions_by_name

    def regions_by_prefix
      ::Multidb.use(:master) do
        ::AwsRegion.includes(:aws_availability_zones).each_with_object({}) do |r, h|
          h[r.prefix] = r
          r.aws_availability_zones.select(&:is_local_zone).each do |az|
            h[az.prefix] = r
          end
        end
      end
    end

    memoize :regions_by_prefix

    def regions_by_id
      ::Multidb.use(:master) { ::AwsRegion.includes(:aws_availability_zones).each_with_object({}) { |r, h| h[r.id] = r } }
    end

    memoize :regions_by_id

    def region(id_or_name)
      # Ok to memoize nil for a missing region since new regions require redeploys
      if id_or_name.is_a?(Numeric) || id_or_name.to_i.positive?
        regions_by_id[id_or_name]
      else
        regions_by_name[id_or_name]
      end
    end

    def zones_by_id
      ::Multidb.use(:master) { ::AwsAvailabilityZone.includes(:aws_region).each_with_object({}) { |z, h| h[z.id] = z } }
    end

    memoize :zones_by_id

    def zones_by_name
      ::Multidb.use(:master) { ::AwsAvailabilityZone.includes(:aws_region).each_with_object({}) { |z, h| h[z.name] = z } }
    end

    memoize :zones_by_name

    def zone_id_to_region(zone_id)
      z = zones_by_id[zone_id]
      if z.nil? && !ttl_valid?(:zones_by_id)
        update_ttl :zones_by_id
        z = zones_by_id(true)[zone_id]
      end
      return nil unless z
      z.aws_region
    end

    def zone(id_or_name)
      if id_or_name.is_a?(Numeric) || id_or_name.to_i.positive?
        z = zones_by_id[id_or_name]
        return z if z || ttl_valid?(:zones_by_id)
        update_ttl :zones_by_id
        az = zones_by_id(true)[id_or_name]
      else
        z = zones_by_name[id_or_name]
        return z if z || ttl_valid?(:zones_by_name)
        update_ttl :zones_by_name
        az = zones_by_name(true)[id_or_name]
      end
      if az.nil? && !id_or_name.nil?
        if id_or_name.include? "pilot"
          ImportRegistry.resolve('statsd').increment 'aws_processing.pilot_availability_zones', tags: %W(az:#{id_or_name})
        else
          ImportRegistry.resolve('alert').error("Missing AZ: #{id_or_name}")
        end
      end
      az
    end

    def instance_type_profiles_by_name
      ::Multidb.use(:master) { ::AwsInstanceTypeProfile.all.each_with_object({}) { |p, h| h[p.api_name] = p } }
    end

    memoize :instance_type_profiles_by_name

    def instance_type(id)
      ::Multidb.use(:master) { ::AwsInstanceType.find(id) }
    end

    memoize :instance_type

    def instance_type_by_key(region_id_or_name, api_name, profile_os, tenancy = 'default')
      if region_id_or_name.is_a?(Numeric) || region_id_or_name.to_i.positive?
        aws_region_id = region_id_or_name
      else
        r = region region_id_or_name
        # Ok to memoize nil when we have a missing region since new regions require redeploys
        return nil unless r
        aws_region_id = r.id
      end
      AwsInstanceType
        .includes(:aws_region, :aws_instance_type_profile)
        .where(
          aws_region_id: aws_region_id,
          api_name: api_name,
          name: profile_os,
          tenancy: tenancy,
          is_current: true
        ).first
    end

    memoize :instance_type_by_key

    def azure_regions_by_name
      ::Multidb.use(:master) { ::AzureRegion.all.each_with_object({}) { |r, h| h[r.name] = r } }
    end

    memoize :azure_regions_by_name

    def azure_regions_by_id
      ::Multidb.use(:master) { ::AzureRegion.all.each_with_object({}) { |r, h| h[r.id] = r } }
    end

    memoize :azure_regions_by_id

    def azure_region(id_or_name)
      # Ok to memoize nil for a missing region since new regions require redeploys
      if id_or_name.is_a?(Numeric) || id_or_name.to_i.positive?
        azure_regions_by_id[id_or_name]
      else
        azure_regions_by_name[id_or_name]
      end
    end

    private

    # Only try to update the cache once every min
    def ttl_valid?(item)
      !@ttls[item].nil? && (::Time.now - @ttls[item]) < 1.minute
    end

    def update_ttl(item)
      @ttls[item] = ::Time.now
    end
  end
end
